@@ -13,5 +13,12 @@ function(type, msg, asError = TRUE)
 
    fun = (if(asError) stop else warning)
 
-   fun( structure(list(message = msg, call = sys.call()), class = c(typeName, "GenericCurlError", "error", "condition")) )
+   fun( structure(list(message = msg, call = sys.call()), class = c(typeName, "GenericCurlError", if(asError) "error" else "warning", "condition")) )
+}
+
+
+getCurlErrorClassNames =
+function()
+{
+  gsub("^CURLE_", "", names(CURLcodeValues))
 }
