@@ -357,7 +357,16 @@
 { "READDATA", CURLOPT_INFILE},
 #endif
 
+#ifdef HAVE_CURLOPT_GSSAPI_DELEGATION
+CINIT(GSSAPI_DELEGATION, LONG, 210),
+#endif
+
+
 #ifdef CURLOPT_FTPLISTONLY
 { "FTPLISTONLY", CURLOPT_DIRLISTONLY}
 #endif
 
+
+
+
+
