@@ -3,7 +3,7 @@
 #include "Rversion.h"
 
 /* Not thread-safe, but okay for now. */
-static char RCurlErrorBuffer[1000] = "<not set>";
+static char RCurlErrorBuffer[1000] = "<error message in RCurl which has not been set>";
 
 #define R_CURL_CHECK_ERROR(status, handle) 	if(status != CURLE_OK) getCurlError(handle, 1, status);
 
@@ -24,13 +24,14 @@ size_t R_curl_read_buffer_callback(void *ptr, size_t size, size_t nmemb, void *s
 typedef struct BufInfo {
     size_t length;
     size_t pos;
-    char *buf;
-    char *cur;
+    void *buf;
+    void *cur;
 } BufInfo;
 
 void * getCurlPointerForData(SEXP el, CURLoption option, Rboolean isProtected, CURL *handle);
 SEXP makeCURLcodeRObject(CURLcode val);
 CURL *getCURLPointerRObject(SEXP obj);
+CURLM *getCURLMPointerRObject(SEXP obj);
 SEXP makeCURLPointerRObject(CURL *obj, int addFinalizer);
 char *getCurlError(CURL *h, int throw, CURLcode status);
 SEXP RCreateNamesVec(const char * const *vals,  int n);
@@ -219,7 +220,7 @@ R_curl_easy_setopt(SEXP handle, SEXP values, SEXP opts, SEXP isProtected, SEXP e
 			status =  curl_easy_setopt(obj, opt, &R_curl_read_buffer_callback);
 			buf->length = Rf_length(el);
 			buf->pos = 0;
-			buf->buf = (char *) RAW(el);
+			buf->buf = RAW(el);
 			buf->cur = buf->buf;
 			status =  curl_easy_setopt(obj, CURLOPT_READDATA, buf);
 		} else if(opt == CURLOPT_READDATA) {
@@ -289,11 +290,9 @@ R_openFile(SEXP r_filename, SEXP r_mode)
     }
     PROTECT(klass = MAKE_CLASS("CFILE"));
     PROTECT(r_ans = NEW(klass));
-    SET_SLOT(r_ans, 
-	     PROTECT(Rf_install("ref")), 
-	     tmp = PROTECT(R_MakeExternalPtr(ans, PROTECT(Rf_install("FILE")), R_NilValue)));
+    SET_SLOT(r_ans, Rf_install("ref"), tmp = R_MakeExternalPtr(ans, Rf_install("FILE"), R_NilValue));
     R_RegisterCFinalizer(tmp, R_closeFILE);
-    UNPROTECT(5);
+    UNPROTECT(2);
     return(r_ans);
 }
 
@@ -448,7 +447,7 @@ addFormElement(SEXP el, SEXP name, struct curl_httppost **post, struct curl_http
 	   type = CHAR(STRING_ELT(VECTOR_ELT(el, 2), 0));
 
 
-        if(GET_LENGTH(VECTOR_ELT(el, 1))) {
+        if(GET_LENGTH(VECTOR_ELT(el, 1))) {  /* the contents field */
 		const char *buf = CHAR(STRING_ELT(VECTOR_ELT(el, 1), 0));
 		if(type) 
   		   curl_formadd(post, last, 
@@ -743,7 +742,7 @@ getCurlError(CURL *h, int throw, CURLcode status)
    if(throw) {
        SEXP e, ptr, fun;
 
-       fun = PROTECT(Rf_findVarInFrame(PROTECT(R_FindNamespace(PROTECT(ScalarString(mkChar("RCurl"))))), PROTECT(Rf_install("curlError"))));
+       fun = Rf_findVarInFrame(R_FindNamespace(ScalarString(mkChar("RCurl"))), Rf_install("curlError"));
 
        PROTECT(e = Rf_allocVector(LANGSXP, 4));
        SETCAR(e, fun); ptr = CDR(e);
@@ -752,7 +751,7 @@ getCurlError(CURL *h, int throw, CURLcode status)
        SETCAR(ptr, ScalarLogical(throw));  ptr = CDR(ptr);
 
        Rf_eval(e, R_GlobalEnv);
-       UNPROTECT(5);
+       UNPROTECT(1);
    }
 
 #endif
@@ -815,6 +814,9 @@ getCurlPointerForData(SEXP el, CURLoption option, Rboolean isProtected, CURL *cu
 	    case INTSXP:
 		    ptr = (void *) malloc(sizeof(long));
 		    *( (long*) ptr) = (long) INTEGER(el)[0];
+  		        /* Don't allow TRUE or 1 for SSL_VERIFYHOST */
+		    if(option == CURLOPT_SSL_VERIFYHOST && INTEGER(el)[0] == 1)
+			*( (long*) ptr) = 2;
 	      break;
 	    case EXTPTRSXP:
 		    ptr = (void *) R_ExternalPtrAddr(el);
@@ -939,7 +941,7 @@ size_t
 R_curl_write_binary_data(void *buffer, size_t size, size_t nmemb, void *userData)
 {
   RCurl_BinaryData *data;
-  unsigned int total = (unsigned int)(size*nmemb);
+  int total = size*nmemb;
   data = (  RCurl_BinaryData *) userData;
   if(!data->data || (data->cursor +  total >= data->data + data->alloc_len )) {
        data->alloc_len = MAX( 2 * data->alloc_len, data->alloc_len + total);
@@ -992,7 +994,7 @@ R_call_R_write_function(SEXP fun, void *buffer, size_t size, size_t nmemb, RWrit
 	PROTECT(str = mkCharLenCE(tmp, len, encoding));
 #else
 
-	PROTECT(str = mkCharLenCE(buffer, (int)(size * nmemb), encoding));
+	PROTECT(str = mkCharLenCE(buffer, size * nmemb, encoding));
 
 #endif
 
@@ -1067,11 +1069,11 @@ checkEncoding(char *buffer, size_t len, RWriteDataInfo *data)
 	SETCAR(e, Rf_install("findHTTPHeaderEncoding"));
 #else
 	PROTECT(ns_name = mkString("RCurl"));
-	ns_env = PROTECT(R_FindNamespace(ns_name));
+	ns_env = R_FindNamespace(ns_name);
 	SETCAR(e, findVarInFrame(ns_env, Rf_install("findHTTPHeaderEncoding")));
-	UNPROTECT(2);
+	UNPROTECT(1);
 #endif
-	SETCAR(CDR(e), ScalarString(mkCharLen(buffer, (int)len)));
+	SETCAR(CDR(e), ScalarString(mkCharLen(buffer, len)));
 	ans = INTEGER(Rf_eval(e, R_GlobalEnv))[0];
 
 	UNPROTECT(1);
@@ -1105,7 +1107,7 @@ R_curl_write_data(void *buffer, size_t size, size_t nmemb, RWriteDataInfo *data)
 
 #include <Rversion.h>
 
-const char *CurlInfoTypeNames[] =  {
+const char  * const CurlInfoTypeNames[] =  {
     "TEXT", "HEADER_IN", "HEADER_OUT",
     "DATA_IN", "DATA_OUT", "SSL_DATA_IN", "SSL_DATA_OUT", 
     "END"
@@ -1183,8 +1185,18 @@ R_curl_progress_callback (SEXP fun, double total, double now, double uploadTotal
 
 	ans = R_tryEval(e, R_GlobalEnv, &errorOccurred);
 
-	if(GET_LENGTH(ans) && TYPEOF(ans) == INTSXP) {
-	    status = INTEGER(ans)[0];
+	if(GET_LENGTH(ans) && (TYPEOF(ans) == INTSXP || TYPEOF(ans) == REALSXP || TYPEOF(ans) == LGLSXP)) {
+	    switch(TYPEOF(ans)) {
+	    case REALSXP:
+		status = (int) REAL(ans)[0];
+		break;
+	    case INTSXP:
+		status = (int) INTEGER(ans)[0];
+		break;
+	    case LGLSXP:
+		status = (int) LOGICAL(ans)[0];
+	    }
+//	    status = INTEGER(ans)[0];
 	}
 	else
 	    status = errorOccurred;
@@ -1229,6 +1241,31 @@ getCURLPointerRObject(SEXP obj)
 	return(handle);
 }
 
+CURLM *
+getCURLMPointerRObject(SEXP obj)
+{
+	CURLM *handle;
+	SEXP ref;
+	if(TYPEOF(obj) != EXTPTRSXP)
+   	   ref = GET_SLOT(obj, Rf_install("ref"));
+	else
+    	   ref = obj;
+
+	handle = (CURLM *) R_ExternalPtrAddr(ref);
+	if(!handle) {
+		PROBLEM "Stale CURLM handle being passed to libcurl"
+		ERROR;
+	}
+
+	if(R_ExternalPtrTag(ref) != Rf_install("MultiCURLHandle")) {
+		PROBLEM "External pointer with wrong tag passed to libcurl. Was %s",
+                        CHAR(PRINTNAME(R_ExternalPtrTag(ref)))
+		ERROR;
+	}
+
+	return(handle);
+}
+
 static void
 R_finalizeCurlHandle(SEXP h)
 {
@@ -1236,15 +1273,41 @@ R_finalizeCurlHandle(SEXP h)
 
    if(curl) {
 #ifdef RCURL_DEBUG_MEMORY
-     fprintf(stderr, "Clearing %p\n", (void *)curl);fflush(stderr);  
+     REprintf("Clearing curl handle %p\n", (void *)curl);fflush(stderr);  
 #endif
-
      CURLOptionMemoryManager *mgr = RCurl_getMemoryManager(curl);
      curl_easy_cleanup(curl);
      RCurl_releaseManagerMemoryTickets(mgr); 
    }
 }
 
+static void
+R_finalizeMultiCurlHandle(SEXP h)
+{
+   CURLM *multi_handle = getCURLMPointerRObject(h);
+
+   if(multi_handle) {
+#ifdef RCURL_DEBUG_MEMORY
+     REprintf("Clearing multi-handle %p\n", (void *)multi_handle);fflush(stderr);  
+#endif
+
+     CURLMsg *msg;
+     int numMsgs = 1;
+
+     while( (msg = curl_multi_info_read(multi_handle, &numMsgs)) ) {
+       curl_multi_remove_handle(multi_handle, msg->easy_handle);
+#if 0
+/* This is problematic. These CURL handles may still be in use in R, i.e. reachable. So we cannot clean them.*/
+       CURLOptionMemoryManager *mgr = RCurl_getMemoryManager(msg->easy_handle);
+       curl_easy_cleanup(msg->easy_handle);
+       RCurl_releaseManagerMemoryTickets(mgr); 
+#endif
+     }
+	 
+   curl_multi_cleanup(multi_handle);
+   }
+}
+
 SEXP
 R_test_finalizeCurlHandle(SEXP h)
 {
@@ -1276,7 +1339,7 @@ makeCURLPointerRObject(CURL *obj, int addFinalizer)
 
 	if(addFinalizer) {
 #ifdef RCURL_DEBUG_MEMORY
-	    fprintf(stderr, "adding finalizer to curl object %p\n", obj);fflush(stderr);
+	    Rprintf("adding finalizer to curl object %p\n", obj);fflush(stderr);
 #endif
 	    R_RegisterCFinalizer(ref, R_finalizeCurlHandle);
 	}
@@ -1356,7 +1419,7 @@ RCurlVersionInfoToR(const curl_version_info_data *d)
    SET_VECTOR_ELT(ans, 3, mkString(d->host));
    SET_VECTOR_ELT(ans, 4, ScalarInteger(d->features)); 
    SET_VECTOR_ELT(ans, 5, mkString(d->ssl_version ? d->ssl_version : ""));
-   SET_VECTOR_ELT(ans, 6, ScalarInteger((int)d->ssl_version_num));
+   SET_VECTOR_ELT(ans, 6, ScalarInteger(d->ssl_version_num));
    SET_VECTOR_ELT(ans, 7, mkString(d->libz_version));
 
    SET_VECTOR_ELT(ans, 8, getRStringsFromNullArray(d->protocols));
@@ -1422,7 +1485,7 @@ getRStringsFromNullArray(const char * const *d)
 
 
 #if 0
-char *DefaultURL = "http://www.omegahat.net/index.html";
+char *DefaultURL = "http://www.omegahat.org/index.html";
 void
 R_test_curl(void)
 {
@@ -1456,10 +1519,11 @@ R_check_bits(int *val, int *bits, int *ans, int *n)
 
 
 
+
 SEXP
 makeMultiCURLPointerRObject(CURLM *obj)
 {
-    SEXP ans, klass;
+    SEXP ans, klass, ref;
 
 	if(!obj) {
 		PROBLEM "NULL CURL handle being returned"
@@ -1469,13 +1533,13 @@ makeMultiCURLPointerRObject(CURLM *obj)
 	
 	PROTECT(klass = MAKE_CLASS("MultiCURLHandle"));
 	PROTECT(ans = NEW(klass));
-	ans = SET_SLOT(ans, PROTECT(Rf_install("ref")), 
-           PROTECT(R_MakeExternalPtr((void *) obj,
-		PROTECT( Rf_install("MultiCURLHandle")),
-	       	R_NilValue)));
-
-	/*XXX R_RegisterCFinalizer(ans, R_finalizeMultiCurlHandle); */
-	UNPROTECT(5);
+	PROTECT(ref = R_MakeExternalPtr((void *) obj, Rf_install("MultiCURLHandle"), R_NilValue));
+	
+Rprintf("registered finalizer for multi %p\n", obj);
+	R_RegisterCFinalizer(ref, R_finalizeMultiCurlHandle);
+	ans = SET_SLOT(ans, Rf_install("ref"), ref);
+	
+	UNPROTECT(3);
 
 	return(ans);
 } 
@@ -1523,7 +1587,7 @@ R_pushCurlMultiHandle(SEXP m, SEXP curl)
 
     status = curl_multi_add_handle(h, c);
 
-    return(makeCURLcodeRObject((int)status));
+    return(makeCURLcodeRObject(status));
 }
 
 SEXP
@@ -1537,7 +1601,7 @@ R_popCurlMultiHandle(SEXP m, SEXP curl)
 
     status = curl_multi_remove_handle(h, c);
 
-    return(makeCURLcodeRObject((int)status));    
+    return(makeCURLcodeRObject(status));    
 }
 
 SEXP
@@ -1596,7 +1660,7 @@ R_curlMultiPerform(SEXP m, SEXP repeat)
 
  
     PROTECT(ans = allocVector(VECSXP, 2));
-    SET_VECTOR_ELT(ans, 0, makeCURLcodeRObject((int)status));
+    SET_VECTOR_ELT(ans, 0, makeCURLcodeRObject(status));
     SET_VECTOR_ELT(ans, 1, ScalarInteger(n));
     UNPROTECT(1);
 
@@ -1668,7 +1732,7 @@ SEXP
 R_global_releaseObject(SEXP obj)
 {
 #if RCURL_DEBUG_MEMORY
-    fprintf(stderr, "releasing %p\n", obj);
+    REprintf(stderr, "releasing %p\n", obj);
 #endif
     Rf_PrintValue(obj);
     R_ReleaseObject(obj);
@@ -1703,3 +1767,13 @@ SEXP makeUTF8String(void *buffer, size_t len, cetype_t encoding)
 #endif
 }
 #endif
+
+
+int
+R_seek(void *instream, curl_off_t offset, int origin)
+{
+    int status;
+    status = fseek((FILE *) instream, (long int) offset, origin);
+    /* check status */
+    return(0);
+}
